package ui.panel
{
	import flash.display.DisplayObject;
	import flash.geom.Point;
	import ui.IListenable;
	import ui.ScrollContainer;
	import ui.Spritesheet;
	import ui.button.LinkedPanelButton;
	import com.greensock.TweenLite;
	import com.greensock.easing.Sine;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;

	/**
	 * Panel
	 * Common side panel.
	 * @author Charlie Chao
	 */
	public class Panel extends Sprite
	{
		/**
		 * Constants
		 */
		public static const PADDING:int = 15;
		public static const TITLE_OFFSET:int = 160;
		public static const TITLE_HEIGHT:int = 48;
		public static const BUTTON_SIZE:int = 32;
		public static const CONTENT_OFFSET:int = TITLE_OFFSET + TITLE_HEIGHT;
		public static const FADE_DURATION:Number = 0.35;
		
		/**
		 * Statics
		 */
		public static var uiSpriteSheet:Spritesheet;		// sprite sheet reference
		public static var imageSpriteSheet:Spritesheet;		// sprite sheet reference
		
		/**
		 * Attributes
		 */
		public var panelWidth:int;
		public var category:int;						// side button category
		protected var content:ScrollContainer;			// buttons scrollable panel
		
		/**
		 * Components
		 */
		protected var title:TextField;
		protected var icon:Bitmap;
		protected var image:Bitmap;
		protected var backButton:LinkedPanelButton;
		protected var closeButton:LinkedPanelButton;
		
		/**
		 * Animation
		 */
		protected var tweenIn:TweenLite;			// fade/slide in animaton
		protected var tweenOut:TweenLite;			// fade/slide out animation
		
		public function Panel(width:int, panelCategory:int, iconName:String = null, imageName:String = null) 
		{
			super();
			panelWidth = width;
			x = -panelWidth - 10;			// sets initial panel location off-screen
			alpha = 0;					// sets initial panel opacity to zero
			category = panelCategory;
			
			// setup animation
			tweenIn = new TweenLite(this, FADE_DURATION, { x: 0, alpha: 1 } );
			tweenIn.paused = true;
			tweenOut = new TweenLite(this, FADE_DURATION, { alpha: 0, onComplete: dispose } );	
			tweenOut.paused = true;
			
			// setup content
			content = new ScrollContainer();
			content.y = Panel.CONTENT_OFFSET;
				
			// setup image
			if (imageName != null) loadImage(imageName);
			
			// setup back button
			backButton = new LinkedPanelButton("", 0, 0, Panel.BUTTON_SIZE, Panel.BUTTON_SIZE, uiSpriteSheet.getSprite("panel_back_alt"), Main.PANEL_NONE);
			closeButton = new LinkedPanelButton("", 0, 0, Panel.BUTTON_SIZE, Panel.BUTTON_SIZE, uiSpriteSheet.getSprite("panel_close_alt"), Main.PANEL_NONE);
			closeButton.bgColor = backButton.bgColor = 0xFFFFFF;
			closeButton.bgAlpha = backButton.bgAlpha = 0.7;
			 backButton.x = PADDING;
			closeButton.x = PADDING * 2 + backButton.width + backButton.x;
			
			// setup icon
			icon = new Bitmap(uiSpriteSheet.getSprite(iconName));
			icon.x = Panel.PADDING;
			icon.y = Panel.TITLE_OFFSET + (Panel.TITLE_HEIGHT - icon.height) / 2;
			addChild(icon);
			
			// setup title text
			title = new TextField();
			title.defaultTextFormat = CommonStyle.PANEL_FORMAT_TITLE;
			//title.embedFonts = true;
			title.autoSize = TextFieldAutoSize.LEFT;
			title.selectable = false;
			title.text = "Panel";
			title.x = Panel.PADDING * 2 + icon.width;
			title.y = Panel.TITLE_OFFSET + TITLE_HEIGHT / 2 - title.height / 2;
			
			//addChild(contentPane);
			addChild(icon);
			addChild(title);
			addChild(backButton);
			addChild(closeButton);
			
			// add initial render event
			addEventListener(Event.ADDED_TO_STAGE, added);
		}
		
		// added to screen handler
		public function added(e:Event = null):void 
		{
			// add linked panel button listeners
			for (var i:int = 0; i < numChildren; i++) {
				var child:DisplayObject = getChildAt(i);
				if (child is IListenable) IListenable(child).listen();
			}
			// fade and render
			fadeIn();
			render();
		}
		
		// renders panel
		public function render():void 
		{
			// remove initial render event
			if (hasEventListener(Event.ADDED)) removeEventListener(Event.ADDED, render);
			
			// draw background
			graphics.clear();
			graphics.beginBitmapFill(uiSpriteSheet.getSprite("panel_pattern"));
			graphics.drawRect(panelWidth, 0, 10, stage.stageHeight);
			graphics.endFill();
			graphics.beginFill(CommonStyle.PANEL_COLOR_BG);
			graphics.drawRect(0, 0, panelWidth, stage.stageHeight);
			graphics.endFill();
			
			// draw title bar
			graphics.beginFill(CommonStyle.PANEL_COLOR_TITLE_BAR);
			graphics.drawRect(0, Panel.TITLE_OFFSET, panelWidth, Panel.TITLE_HEIGHT);
			graphics.endFill();
			
			// update internal elements
			if (contains(content))
				content.bound = new Point(panelWidth, stage.stageHeight - Panel.CONTENT_OFFSET);
			//closeButton.x = panelWidth - Panel.BUTTON_SIZE;
		}
		
		// fades in panel
		public function fadeIn():void 
		{
			tweenOut.pause();
			tweenIn.restart();
		}
		
		// fades out panel
		public function fadeOut():void
		{
			tweenIn.pause();
			tweenOut.restart();
		}
		
		// sets previous panel
		public function set previousPanel(panel:int):void {
			backButton.linkedPanel = panel;
		}
		
		// returns previous panel
		public function get previousPanel():int {
			return backButton.linkedPanel;
		}
		
		// loads image to screen
		public function loadImage(imageName:String):void 
		{
			if (image != null && contains(image)) removeChild(image);
			image = new Bitmap(imageSpriteSheet.getSprite(imageName));
			addChild(image);
		}
		
		// overridable function for initializing buttons, which requires the panel list to be filled in Main to be called
		public function constructButtons(panels:Vector.<Panel>):void { }
		
		// overridable function for updating panel
		public function update():void { }
		
		// removes the panel from screen and removes all the listeners
		protected function dispose():void 
		{
			// add linked panel button listeners
			for (var i:int = 0; i < numChildren; i++) {
				var child:DisplayObject = getChildAt(i);
				if (child is IListenable) IListenable(child).unlisten();
			}
			// remove panel from parent
			if (parent != null)	parent.removeChild(this);
		}
		
	}

}